<?php
/**
 * This is the entrypoint for the locrMAPS for WooCommerce WordPress plugin.
 *
 * @package locrMAPS for WooCommerce
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMapsForWooCommerce\DDD\Services;

use Locr\WordPress\Plugin\LocrMapsForWooCommerce\DDD\ValueObjects\ImageType;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the image.
 *
 * @property-read string    $data The data.
 * @property-read ImageType $type The type.
 */
final class Image {
	/**
	 * This function initializes the image.
	 *
	 * @param string    $data The data.
	 * @param ImageType $type The type.
	 */
	public function __construct( private string $data, private ImageType $type ) {
	}

	/**
	 * This function gets the data.
	 *
	 * @param string $name The name.
	 */
	public function __get( string $name ): mixed {
		return match ( $name ) {
			'data' => $this->data,
			'type' => $this->type,
			default => null
		};
	}

	/**
	 * This function loads the image from a data url.
	 *
	 * @param string $data_url The data URL.
	 * @throws \Exception If the data URL is invalid.
	 */
	public static function from_data_url( string $data_url ): self {
		$comma_position = strpos( $data_url, ',' );
		if ( false === $comma_position ) {
			throw new \Exception( 'The data URL is invalid (no comma found).' );
		}
		if ( 128 <= $comma_position ) {
			throw new \Exception( 'The data URL is invalid (comma position is too high).' );
		}
		$pre_data = substr( $data_url, 0, $comma_position + 1 );

		if ( ! preg_match( '/^data:image\/(?P<image_type>png|jpe?g);base64,/', $pre_data, $data_url_match ) ) {
			throw new \Exception( 'The data URL is invalid (no valid image type found).' );
		}

		$data = base64_decode( substr( $data_url, $comma_position + 1 ) );

		return new self(
			data: $data,
			type: ImageType::from_string( $data_url_match['image_type'] )
		);
	}

	/**
	 * This function loads the image from a file.
	 *
	 * @param string $filename The filename.
	 * @throws \Exception If the file does not exist or could not be read.
	 */
	public static function from_file( string $filename ): self {
		if ( ! file_exists( $filename ) ) {
			throw new \Exception( 'The file does not exist.' );
		}

		$image_data = file_get_contents( $filename );
		if ( false === $image_data ) {
			throw new \Exception( 'The file could not be read.' );
		}

		return new self(
			data: $image_data,
			type: ImageType::from_string( pathinfo( $filename, PATHINFO_EXTENSION ) )
		);
	}

	/**
	 * This function resizes the image.
	 *
	 * @param int $max_length The maximum length.
	 * @throws \Exception If the GD library is not available.
	 */
	public function resize_by_max_length( int $max_length ): self {
		if ( ! class_exists( 'GdImage' ) ) {
			throw new \Exception( 'The GD library is not available.' );
		}

		$image = imagecreatefromstring( $this->data );
		if ( false === $image ) {
			throw new \Exception( 'The image could not be created.' );
		}
		$image_width  = imagesx( $image );
		$image_height = imagesy( $image );
		$image_ratio  = $image_width / $image_height;
		if ( $image_ratio >= 1 ) {
			$resized_width  = (int) min( $max_length, $image_width );
			$resized_height = (int) ( $resized_width / $image_ratio );
		} else {
			$resized_height = (int) min( $max_length, $image_height );
			$resized_width  = (int) ( $resized_height * $image_ratio );
		}
		$resized_image = imagescale( $image, $resized_width, $resized_height );
		if ( false === $resized_image ) {
			throw new \Exception( 'The image could not be resized.' );
		}
		$tmp_filename        = wp_tempnam();
		$resize_successfully = match ( $this->type ) {
			ImageType::JPEG => imagejpeg( $resized_image, $tmp_filename ),
			ImageType::PNG  => imagepng( $resized_image, $tmp_filename )
		};
		if ( ! $resize_successfully ) {
			throw new \Exception( 'The image could not be saved.' );
		}

		$resized_image_data = file_get_contents( $tmp_filename );
		if ( false === $resized_image_data ) {
			throw new \Exception( 'The image could not be read.' );
		}

		wp_delete_file( $tmp_filename );

		return new self(
			data: $resized_image_data,
			type: $this->type
		);
	}

	/**
	 * This function converts the image to a data URL.
	 */
	public function to_data_url(): string {
		$base64_data = base64_encode( $this->data );
		return "data:image/{$this->type->name};base64,{$base64_data}";
	}
}
